/*
reverse.c
Copyright (C) yyyy Ruslan Soldatov

* RNASurface: fast and accurate detection of locally optimal potentially structured RNA segments

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
      
#define handle_error(msg) \
do { perror(msg); exit(EXIT_FAILURE); } while (0)

main(int argc,char* argv[])
{
	FILE* out;
	char c[1];
	char* addr;
	struct stat sb;
	int fd,i,len=1;
	char* namec;
	
	if (argc!=2)
	{
		fprintf(stderr,"wrong command line parameters");
		exit(1);
	}
	

	namec=(char*)malloc( (strlen(argv[1])+2)*sizeof(char) );
	sprintf(namec,"%sc",argv[1]);

	fprintf(stderr,"argv: %s\n",argv[1]);
	fprintf(stderr,"argv len: %d\n",strlen(argv[1]));
	fprintf(stderr,"namec: %s\n",namec);
	fprintf(stderr,"namec length: %d\n",strlen(namec));


	fd=open(argv[1],O_RDONLY);
	out=fopen(namec,"w");
	
	if (out==NULL)
	{
		fprintf(stderr,"wrong output file name");
		exit(1);
	}
	
	if (fd == -1)
         handle_error("open");

	if (fstat(fd, &sb) == -1)   /* To obtain file size */
        handle_error("fstat");
	
	
		  
	read(fd,c,1);
	if ( c[0]!='>')
	{
		fprintf(stderr,"input sequence in fasta format");
		exit(1);
	}
	fprintf(out,">");
	while ( c[0]!='\n' )
	{
		read(fd,c,1);
		fprintf(out,"%c",c[0]);
		len++;
	}	
	len--;
	
	addr = mmap(NULL, sb.st_size, PROT_READ,MAP_PRIVATE, fd, 0); 
	if (addr == MAP_FAILED)
		 handle_error("mmap");

	for (i=sb.st_size-1;i>=len;i--)	
	{
		if ( addr[i]=='a' || addr[i]=='A' ) fprintf(out,"t");
		else if ( addr[i]=='t' || addr[i]=='T' || addr[i]=='u' || addr[i]=='U' ) fprintf(out,"a");
		else if ( addr[i]=='g' || addr[i]=='G' ) fprintf(out,"c");
		else if ( addr[i]=='c' || addr[i]=='C' ) fprintf(out,"g");
		else if ( addr[i]!='\n' && addr[i]!='\r' ) fprintf(out,"%c",addr[i]);
	}
	fclose(out);
	
}