/*
coef.c
Copyright (C) yyyy Ruslan Soldatov

* RNASurface: fast and accurate detection of locally optimal potentially structured RNA segments

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include "coef.h"
#include "rnaSurface.h"

int lSum=6600;
int lsqSum=7288000;
int lN=6;

/* 
	Define approximation coefficients
*/
void read_coefficients(char* headFile)
{
	int i,j,k,u;
	double x;
	char str[10000];
	char str1[10000];
	char root0[1000],root[1000];
	char coefPath[1000];
	int len0,len;

	sprintf(coefPath,"%s%csource%cqcoefficients",headFile,DIS,DIS);

	//FILE *f=fopen("./source/qcoefficients","r");
	FILE *f;
	if ( (f=fopen(coefPath,"r")) == NULL )
	{
		fprintf(stderr,"Error in opening file with coefficients for read\n");
		exit(18);
	}
	i=-1;j=0;
	
	while ( (i<control_points) && (feof(f)==0) )
	{
		fgets(str,10000,f);
		if (str[0]=='>')
		{
				i++;
				j=0;
				sscanf(str,">%d",&u);
				coef[i].length=u;
		}
		else if (j<27)
		{
			for (k=0;k<256;k++)
			{
				sscanf(str,"%lf ",&x);
				coef[i].mean[j][k]=x;
				if (strchr(str,' ') != NULL ) 
				{
					sprintf(str1,"%s",strchr(str,' ')+1);
					strcpy(str,str1);
				}
			}
			sscanf(str,"%lf",&x);
			coef[i].mean[j][k]=x;
			fgets(str,10000,f);
			for (k=0;k<256;k++)
			{
				sscanf(str,"%lf ",&x);
				coef[i].sd[j][k]=x;
				if (strchr(str,' ') != NULL ) 
				{
					sprintf(str1,"%s",strchr(str,' ')+1);
					strcpy(str,str1);
				}
			}
			sscanf(str,"%lf",&x);
			coef[i].sd[j][k]=x;			
			j++;
		}
		
	}
	fclose(f);
}

/* 
	Define control points lower and upper bounds based on minimum and maximum parameters of window 
*/
void define_bounds(int min,int max)
{
	int i;
	
	if (min<=coef[0].length) lower_ind=0;
	else if (min>=coef[5].length) lower_ind=5;
	else 
	{
		for (i=0;i<control_points-1;i++)
		{
			if (coef[i].length<=min && coef[i+1].length>min) lower_ind=i;
		}
	}
	if (lower_ind>1) lower_ind=1;
	
	if (max<=coef[0].length) upper_ind=1;
	else if (max>=coef[control_points-1].length) upper_ind=control_points-1;
	else 
	{
		for (i=0;i<control_points-1;i++)
		{
			if (coef[i].length<max && coef[i+1].length>=max) upper_ind=i+1;
		}
	}
	if (upper_ind<5) upper_ind=5;
	
}

double meanEst(int length,double mean[])
{
	int i,i0=5;
	double a=0,b=0;
	
	if (length>coef[control_points-1].length)
	{
		for (i=i0;i<control_points;i++) a+=mean[i]*(coef[i].length-lSum/lN);
		a=a/(lsqSum-lSum*lSum/lN);
		for (i=i0;i<control_points;i++) b+=mean[i];
		b-=a*lSum;
		b=b/lN;
		return (a*length+b);
	}
	else if (length>coef[0].length)
	{
		for (i=0;i<control_points-1;i++)
		{
			if (coef[i].length<=length && coef[i+1].length>=length)
			{
				return (mean[i]*(coef[i+1].length-length)+mean[i+1]*(length-coef[i].length))/(coef[i+1].length-coef[i].length);
			}
		}
	}
	else if (length>=length_low_bound)
	{
		return (mean[0]*(coef[1].length-length)+mean[1]*(length-coef[0].length))/(coef[1].length-coef[0].length);
	}
	
	return -1;
}



double sdEst(int length,double sd[])
{
	int i;
	int i1=1,i2=5;
	double c,alpha,k;
	if (length_low_bound<=length && coef[1].length>=length)
	{
		return (sd[0]*(coef[1].length-length)+sd[1]*(length-coef[0].length))/(coef[1].length-coef[0].length);
	}
	else
	{
		k=(sd[i2]/sd[i1])*(sd[i2]/sd[i1]);
		alpha=(coef[i2].length-coef[i1].length*k)/(k-1);
		c=sd[i1]/sqrt(coef[i1].length+alpha);
		return c*sqrt(length+alpha);
	}
}


double meanEst_small(int length,double mean[])
{
	int i;
	for (i=0;i<control_points-1;i++)
	{
		if (coef[i].length<=length && coef[i+1].length>=length)
		{
			return (mean[i]*(coef[i+1].length-length)+mean[i+1]*(length-coef[i].length))/(coef[i+1].length-coef[i].length);
		}
	}
	//printf("lengths: %d %d %d %d %d\n",length,coef[0].length,coef[1].length,coef[2].length,coef[3].length);
	return -1;
}


double sdEst_small(int length,double sd[])
{
	int i;
	for (i=0;i<control_points-1;i++)
	{
		if (coef[i].length<=length && coef[i+1].length>=length)
		{
			return (sd[i]*(coef[i+1].length-length)+sd[i+1]*(length-coef[i].length))/(coef[i+1].length-coef[i].length);
		}
	}
	return -1;
}

/*
main(int argc,char* argv[])
{
	double m[11]={60,80,160,300,600,1000,1040,1080,1120,1160,1200};
	double sd[11]={7.75,8.94,12.65,17.32,24.5,31.62,32.25,32.86,33.47,34.06,34.64};
	int k,len=atoi(argv[1]);
	read_coefficients();
	k=control_points;
	printf("mean %lf\n",meanEst(len,m));
	printf("sd %lf %lf\n",sdEst(len,sd),sdEst(len,sd)*sdEst(len,sd));
}
*/
/*
main()
{
	int min,max;
	scanf("%d\n%d",&min,&max);
	read_coefficients();
	printf("%d\t%d\n",min,max);
	define_bounds(min,max);
	printf("%d\t%d\n",lower_ind,upper_ind);
}
*/