/*
cumulativeLocOpt.c
Copyright (C) yyyy Ruslan Soldatov

* Fast and accurate identification of locally optimal motfis with high structural potential

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
      
#define handle_error(msg) \
do { perror(msg); exit(EXIT_FAILURE); } while (0)

#define mmap_size 200000000;

struct unit
{
	int pos;
	float value;
};

struct unitLO
{
	int start;
	int end;
	float energy;
	float z;
};


int intersectLen(int start,int end,int pos,int win)
{
	int len=1;
	if ( pos+win < end ) len+=pos+win; else len+=end;
	if ( pos-win > start ) len-=pos-win; else len-=start;
	return len;
}

makeEmptyFile(char* out,int length_byte)
{
	FILE* f=fopen(out,"wb");
	int i;
	for (i=0;i<length_byte;i++) fwrite("",1,1,f);
	fclose(f);
}

/*
	Description: calculate density of locally optimal structures
	in - input binary file with locally optimal structures
	output - output binary density file
	seq_length - length of the sequence
*/
void densityLocOpt(FILE* in,char* out,int seq_length,int win)
{
	int elementSize,fd,i,pos,middle,skip,skipBytes,psize;
	size_t size;
	struct unit* addr,*addr1;
	struct unitLO locOpt;
	elementSize=sizeof(struct unit);
	psize=getpagesize();
	size=mmap_size;
	size=size-size % psize;
	if ( size > seq_length*elementSize ) size=seq_length*elementSize;
	fseek (in,0,SEEK_SET);
	makeEmptyFile(out,seq_length*elementSize+psize-(seq_length*elementSize)%psize);
	
	fd=open(out,O_RDWR | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR);
	
	if (fd == -1)
         handle_error("open");	
	
	/*
	for (i=0;i<seq_length*elementSize;i++)	
		write (fd,"",1);
	*/

	lseek (fd, 0, SEEK_SET);
		
	skip=0;skipBytes=0;
	//fprintf(stderr,"\nmapping file with skipBytes=%d and size=%d; file size=%d; psize=%d\n",skipBytes,size,seq_length*elementSize,psize);
	addr = mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fd, skipBytes); 
	if (addr == MAP_FAILED)
		 handle_error("mmap");
	
	for (i=0;i<size/elementSize;i++)
	{
		addr[i].pos=i+1;
		addr[i].value=0;
	}
	
	while ( fread(&locOpt,sizeof(struct unitLO),1,in)!=0 )
	{
		if ( locOpt.end > seq_length )
		{
			fprintf(stderr,"Locally optimal structure with coordinate (%d,%d) out of sequence range",locOpt.start,locOpt.end);
			exit(1);
		}
		if ( (skipBytes+size<seq_length*elementSize) && (locOpt.end>skip+(size-psize)/elementSize) ) 
		{
			skip+=(size-psize)/elementSize;
			skipBytes+=size-psize;
			munmap(addr,size);
			//fprintf(stderr,"mapping file with skipBytes=%d and size=%d; file size=%d; psize=%d\n",skipBytes,size,seq_length*elementSize,psize);
			if (size>seq_length*elementSize-skipBytes){
				size=seq_length*elementSize-skipBytes;
			}
			addr=mmap(0,size, PROT_READ | PROT_WRITE,MAP_SHARED, fd, skipBytes); 
			if (addr == MAP_FAILED)
				handle_error("mmap");	
			for (i=psize/elementSize;i<size/elementSize;i++)
			{
				addr[i-1].pos=skip+i;
				addr[i-1].value=0;
			}
		}
		{
			middle=(locOpt.start+locOpt.end)/2;
			for (i=middle-win;i<=middle+win;i++)
			{
				if (i-skip>=1 && i-skip<size/elementSize)
				{
					if (addr[i-1-skip].value == 0 )
					{
						addr[i-1-skip].pos=i;
						addr[i-1-skip].value=(locOpt.z)*(locOpt.z)/win    		/**intersectLen(locOpt.start,locOpt.end,i,win)/(locOpt.end-locOpt.start+1)*/;
					}
					else
					{
						addr[i-1-skip].value+=(locOpt.z)*(locOpt.z)/win   		/**intersectLen(locOpt.start,locOpt.end,i,win)/(locOpt.end-locOpt.start+1)*/;
					}
				}
			}
		}

	}
	munmap(addr,size);
	close(fd);
}
/*
main(int argc,char* argv[])
{
	FILE* in=fopen(argv[1],"rb");
	densityLocOpt(in,argv[2],2200000,500);
}
*/