/*
estimation.h
Copyright (C) yyyy Ruslan Soldatov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/

#include "estimation.h"
#include "../shuffle/shuffle.h"
#include "../shuffle/methods.h"
#include "../shuffle/transformation.h"
#include "../rnaslider/rnaSlider.h"
#define outlier 10000000

int estimateTrials(int seqlen, int winmax, int samplesize, double intersect){
	int k;
	
	/* number of subsequences in one sequence */
	k= (int)((double)(seqlen-winmax)/winmax/(1-intersect)+1);
	return (int)( (double)samplesize/k ) + 1;
	
}

struct Params* estimateParam(char* seq, double* paired, double* unpaired, int winmax, int winmin, int samplesize, double intersect, int di, int markov, char* argv){
	struct Params* par;
	int seqlen, i,k, trials;
	struct shuffleSeq shuf;
	double *mean,*sd;
	int *size;
	
	
	seqlen=strlen(seq);
	trials=estimateTrials(seqlen, winmax, samplesize, intersect);
	printf("shufflings: %d\n",trials);
	srand(time(NULL));
	
	mean=(double*)malloc( (winmax-winmin+1) *sizeof(double));
	sd=(double*)malloc( (winmax-winmin+1) *sizeof(double));
	size=(int*)malloc( (winmax-winmin+1) *sizeof(double));
	for (i=winmin;i<winmax+1;i++){
		mean[i-winmin]=0;
		sd[i-winmin]=0;
		size[i-winmin]=0;
	}
	
	/* - do while i!=truials and i++ if energy!=na !!!!!!!!!!!!!!!!!!!!!!
	   - check why energy == na and params=na at length=83
	 */
	i=0;
	while (i<trials){
        shuf = shuffle(seq, paired, unpaired, di, markov);
		if (paired==NULL || unpaired==NULL)
			runSlider(shuf.seq,paired,unpaired,winmax,winmin,(1-intersect),argv);	
		else 
			runSlider(shuf.seq,shuf.data,shuf.data1,winmax,winmin,(1-intersect),argv);
		//printf("auto %lf %lf, %lf %lf\n",avto(paired,seqlen,1),avto(shuf.data,seqlen,1),avto(paired,seqlen,2),avto(shuf.data,seqlen,2));
		/*
		for (k=0;k<seqlen;k++){
            fprintf(stderr,"%d %lf %lf, ",k,shuf.data[k],shuf.data1[k]);
        }
		printf("\n");*/
        //free(shuf.seq);free(shuf.data);free(shuf.data1);
		for (k=winmin;k<winmax+1;k++){
			if ( abs(expParam[k-winMin]) >= outlier || abs(sdParam[k-winMin]) >= outlier )
				i--;
			else{
				mean[k-winMin]+=expParam[k-winMin];
				sd[k-winMin]+=sdParam[k-winMin];
				size[k-winMin]+=sizeParam[k-winMin];
			}
		}
		i++;
		free(expParam);free(sdParam);free(sizeParam);
	}
	
	for (i=winmin;i<winmax+1;i++){
		mean[i-winmin]/=size[i-winmin];
		sd[i-winmin]/=size[i-winmin];
		sd[i-winmin]-=mean[i-winmin]*mean[i-winmin];
		sd[i-winmin]=sqrt(sd[i-winmin]);
	}
	
	par=(struct Params*)malloc( (winmax-winmin+1)*sizeof(struct Params) );
	for (i=winmin;i<winmax+1;i++){
		par[i-winMin].length=i;
		par[i-winMin].mean=mean[i-winmin];
		par[i-winMin].sd=sd[i-winmin];
	}

	return par;
}