/*
main.c
Copyright (C) yyyy Ruslan Soldatov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cmdline.h"
#include "../shuffle/shuffle.h"
#include "../shuffle/shuffleFunc.h"
#include "estimation.h"

/* return sequence length */
int len(FILE* f)
{
	int i=0;
	char c;
	while ( feof(f)==0 )
	{
		c=fgetc(f);
		if ( convert(c)!=-1 ) i++;
		else if ( c!='\n' && c!='\r' && c!=' ' && c!='\t' && c!=-1 ) i++;
	}
	return i;
}


char* read_seq(FILE* file){
	char c;
	char* tmp;
	int length,i;
	fpos_t f_pos;
	
	if ( (c=fgetc(file)) != '>' ){
		fprintf(stderr,"Incorrect fasta-file");
		exit(1);
	}
	while ( (c=fgetc(file)) != '\n' && c!='\r' )
		;
	fgetpos(file,&f_pos);
	length=len(file);
	fsetpos(file,&f_pos);
	tmp=(char*)malloc( (length+1)*sizeof(char));
	i=0;
	while ( feof(file)==0 ){
		c=fgetc(file);
		if ( convert(c)!=-1 || (c!='\n' && c!='\r' && c!=' ' && c!='\t' && c!=-1) ){
			tmp[i]=c;
			i++;
		}
	}
	tmp[i]='\0';
	return tmp;
}


double* read_data(FILE* file,int length){
	double* tmp;
	int i;
	double x;
	tmp=(double*)malloc(length*sizeof(double));
	for (i=0;i<length;i++)
		tmp[i]=-1;
	while ( feof(file)==0 ){
		fscanf(file,"%d %lf",&i,&x);
		tmp[i-1]=x;
	}
	return tmp;
}

int main(int argc, char* argv[]){
	char *iname_seq,*iname_paired,*iname_unpaired;
	char *outname;
	FILE *inp_seq,*inp_paired,*inp_unpaired,*output;
	int di,markov,i;
	int winmax,winmin,size;
	double intersect;
	char* seq;
	double *paired=NULL,*unpaired=NULL;
	struct gengetopt_args_info args_info;
	struct Params parameters;
	
	di=0;
	markov=0;
	winmin=10;
	winmax=300;
	intersect=0;
	size=100;
	
	if (cmdline_parser(argc, argv, &args_info) != 0)
    {
		fprintf(stderr,"Wrong command line options\n");
		exit(1);
	}
	if ( args_info.input_given )
		iname_seq=args_info.input_arg;
	if ( args_info.paired_given )
		iname_paired=args_info.paired_arg;
	if ( args_info.unpaired_given )
		iname_unpaired=args_info.unpaired_arg;
	if ( args_info.output_given )
		outname=args_info.output_arg;
	if ( args_info.winmax_given )
		winmax=args_info.winmax_arg;
	if ( args_info.winmin_given )
		winmin=args_info.winmin_arg;
	if ( args_info.intersect_given )
		intersect=args_info.intersect_arg;
	if ( args_info.size_given)
		size=args_info.size_arg;
	if ( args_info.di_given ) 
		di=1;
	if ( args_info.markov_given )
		markov=1;	
	if ( args_info.no_transform_given )
		qtransform=0;
	else qtransform=1;
	
	if ( (inp_seq = fopen(iname_seq, "r" )) == NULL ){
		fprintf(stderr,"Error in opening input file with sequence for read\n");
		exit( 1 );
	}	
	if ( args_info.paired_given & (inp_paired = fopen(iname_paired, "r" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file with paired probabilities for read\n");
		exit( 1 );
	}	
	if ( args_info.unpaired_given & (inp_unpaired = fopen(iname_unpaired, "r" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file with unpaired probabilities for read\n");
		exit( 1 );
	}
	if ( args_info.output_given & (output = fopen(outname, "w" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file for write\n");
		exit( 1 );
	}

	/* check command line parameters */
	if (winmin<=0){
		fprintf(stderr,"Winmin must be greater than 0\n");
		exit(1);
	}
	if (winmin>winmax){
		fprintf(stderr,"Winmax must be greater than winmin\n");
		exit(1);
	}
	if ( intersect>1 || intersect<0 ){
		fprintf(stderr,"Intersect must be in range [0,1]\n");
		exit(1);
	}
	if (size<=0){
		fprintf(stderr,"Sample size must be greater than 0\n");
		exit(1);
	}
	if (size<100)
		fprintf(stdout,"Recommended sample size is greater than 100 ...\n");
	
	
	/* read sequence and data */
	seq=read_seq(inp_seq);
	if ( args_info.paired_given )
		paired=read_data(inp_paired,strlen(seq));
	if ( args_info.unpaired_given )
		unpaired=read_data(inp_unpaired,strlen(seq));
		
	if ( winmax > strlen(seq) )
		winmax=strlen(seq);
	if ( winmin > strlen(seq) )
		winmin=strlen(seq);
		
	struct Params* par=estimateParam(seq, paired, unpaired, winmax, winmin, size, intersect, di, markov, argv[0]);
	if ( !args_info.output_given )
		output=stdout;
	for (i=winmin;i<winmax+1;i++)
		fprintf(output,"%d\t%lf\t%lf\n",par[i-winmin].length,par[i-winmin].mean,par[i-winmin].sd );
	
	/* print sequence */
	/*
	if ( !args_info.output_given )
		output=stdout;
	fprintf(output,">shuffle\n");
	for (i=0;i<strlen(seq);i++)
		fprintf(output,"%c",seq[i]);
	fprintf(output,"\n");
	*/
	/* print data */
	/*
	printf("printing");
	for (i=0;i<strlen(seq);i++)
		fprintf(output,"%d %lf\n",(i+1),data[i]);
	fclose(output);
	*/
	
	if ( !args_info.output_given )
		fclose(output);
	if ( args_info.paired_given )
		fclose(inp_paired);
	if ( args_info.unpaired_given )
		fclose(inp_unpaired);
	if ( args_info.input_given )
		fclose(inp_seq);

	return 0;
}

