/*
estimation.h
Copyright (C) yyyy Ruslan Soldatov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/

#include "approx_sample.h"
#include "../rnaslider/rnaSlider.h"
#include "cmdline_approx_sample.h"
#include "estimation.h"



int main(int argc, char* argv[]){
	char *iname_seq,*iname_paired,*iname_unpaired,*outname;
	FILE *inp_seq,*inp_paired,*inp_unpaired,*output;
	char* seq;
	int i, n, winmin=20, winmax=300;
	int sample_size=100;
	double param_p=0.01;
	double a,b;
	struct energy_sample* smpl;
	double *paired=NULL,*unpaired=NULL;
	struct gengetopt_args_info args_info;

	if (cmdline_parser(argc, argv, &args_info) != 0)
    {
		fprintf(stderr,"Wrong command line options\n");
		exit(1);
	}
	if ( args_info.input_given )
		iname_seq=args_info.input_arg;
	if ( args_info.paired_given )
		iname_paired=args_info.paired_arg;
	if ( args_info.unpaired_given )
		iname_unpaired=args_info.unpaired_arg;
	if ( args_info.output_given )
		outname=args_info.output_arg;
	if ( args_info.winmax_given )
		winmax=args_info.winmax_arg;
	if ( args_info.winmin_given )
		winmin=args_info.winmin_arg;
	if ( args_info.size_given)
		sample_size=args_info.size_arg;	
	if ( args_info.par_given )
		param_p=args_info.par_arg;	
	
	if ( (inp_seq = fopen(iname_seq, "r" )) == NULL ){
		fprintf(stderr,"Error in opening input file with sequence for read\n");
		exit( 1 );
	}	
	if ( args_info.paired_given & (inp_paired = fopen(iname_paired, "r" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file with paired probabilities for read\n");
		exit( 1 );
	}	
	if ( args_info.unpaired_given & (inp_unpaired = fopen(iname_unpaired, "r" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file with unpaired probabilities for read\n");
		exit( 1 );
	}
	if ( args_info.output_given & (output = fopen(outname, "w" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file for write\n");
		exit( 1 );
	}	
	
	/* read sequence and data */
	
	seq=read_seq(inp_seq);
	if ( args_info.paired_given )
		paired=read_data(inp_paired,strlen(seq));
	if ( args_info.unpaired_given )
		unpaired=read_data(inp_unpaired,strlen(seq));
		
	if ( winmax > strlen(seq) )
		winmax=strlen(seq);
	if ( winmin > strlen(seq) )
		winmin=strlen(seq);
		
	if ( !args_info.output_given )
		output=stdout;
	
	
	
	n=strlen(seq);
	smpl=(struct energy_sample*)malloc( sample_size*sizeof(struct energy_sample));
	sample_energies(seq, paired, unpaired, n, winmin, winmax, sample_size, param_p, smpl,argv[0]);
	//sample_energies_general( iname_seq, iname_paired, iname_unpaired, winmin, winmax, sample_size, param_p, smpl, argv[0]);

	
	estimateRegPar(smpl,sample_size,&a, &b);
	for (i=0;i<sample_size;i++)
		fprintf(output,"%d %d %d %lf %lf %lf\n",smpl[i].start,smpl[i].end,smpl[i].length,smpl[i].energy,smpl[i].energy_probing,smpl[i].probed_bases);
			
		
	if ( !args_info.output_given )
		fclose(output);
	if ( args_info.paired_given )
		fclose(inp_paired);
	if ( args_info.unpaired_given )
		fclose(inp_unpaired);
	if ( args_info.input_given )
		fclose(inp_seq);
		
	return 0;
	
}
