/*
inferStructure.c
Copyright (C) yyyy Ruslan Soldatov

* RNASurface: fast and accurate detection of locally optimal potentially structured RNA segments

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include "inferStructure.h"
#include "../rnaslider/rnaSlider.h"
#include "../rnaslider/common.h"
#include "../rnaslider/energyfiles/energy.h"
#include "../rnaslider/energyfiles/energy_read.h"
#include "cmdline.h"


int main(int argc, char* argv[]) 
{
	int i, k, l, cands=0;
	int ts,t;
	int length;
	int upperBoundSize=-1;///// set as the file size if not set differently
	char* str;
	int iterations=0;
	char ch;
	int maxLength=0;
	double avgLength=0;
	double start_time,end_time,start_t,end_t;
	double energy;
	char* seqName[100];
	int endPoint;
	double pentr,uentr;
	FILE *inp, *outp;
	char* inFilename;
	char* outFilename;
	
	char* pprobFilename = NULL;
	char* uprobFilename = NULL;
	char* reactFilename = NULL;
	winSize=-1;winPoint=0;

	/*** read parameters from command line trwe***/
	struct gengetopt_args_info args_info;
	
	if (cmdline_parser(argc, argv, &args_info) != 0)
    {
		fprintf(stderr,"Wrong command line options\n");
		exit(1);
	}
	if ( args_info.input_given ) inFilename=args_info.input_arg;
	if ( args_info.output_given ) outFilename=args_info.output_arg;
	if ( args_info.uppersize_given ) upperBoundSize=args_info.uppersize_arg;	
	if ( args_info.traceback_given ) TRACEBACK=1;	
	if ( args_info.winsize_given ) winSize=args_info.winsize_arg;	
	if ( args_info.paired_given ) pprobFilename=args_info.paired_arg;
	if ( args_info.unpaired_given ) uprobFilename=args_info.unpaired_arg;
	if ( args_info.reactivity_given ) reactFilename=args_info.reactivity_arg;
	if ( args_info.weight_given ) exp_const=args_info.weight_arg;

	/******/
	
	char path[PATH_LEN];	
	if ( strcmp(PLATFORM,"win") ==0 )
		GetModuleFileName( NULL, path, PATH_LEN + 1 );
	else
		realpath (argv[0], path);

	char* head=(char*)malloc(PATH_LEN*sizeof(char));
	pathHead(path,head);
	read_tables(head);
	free(head);
	
	
	if( (inp = fopen(inFilename, "r" )) == NULL )	{
		printf("Error in openning input file for read\n");
		exit( 140 );
	}

	if (upperBoundSize==-1)
	{
		fseek (inp, 0L, SEEK_END);
		upperBoundSize = ftell (inp);
		fseek (inp, 0L, SEEK_SET);
	}
	
	str = (char*)malloc(upperBoundSize*sizeof(char));
	
	if( (outp = fopen(outFilename, "w" )) == NULL )	{
		printf("Error in openning input file for write\n");
		exit( 140 );
	}

	ch = fgetc( inp );
	
	// get sequence name
	if (ch == '>') {
		fscanf(inp, "%s",seqName);
	}

	ch = fgetc( inp );
	while ((ch != '\n') && (feof(inp) == 0)) {
		ch = fgetc( inp );
	}
	length = 0;
	
	
	for(i=0; feof(inp) == 0;)
	{
		switch (ch) {
			case 'A': 
			case 'a': 
			case 'C': 
			case 'c': 
			case 'G':
			case 'g': 
			case 'T': 
			case 't':
			case 'U':
			case 'u': ++length;
				str[i] = ch;++i;break;
		}
		ch = fgetc( inp );
	}
	str[i]='\0';
	if (length > 0) {
	
		if (winSize==-1)
			winSize=length;
		endPoint = length - winSize + 1;


		/* Initialization of arrays with experimantal data */		
		uprob_exist=0;pprob_exist=0;react_exist=0;
		if ( uprobFilename!=NULL ){
			initialize_exp(uprobFilename,'u',length);
		}
		if ( pprobFilename!=NULL ){
			initialize_exp(pprobFilename,'p',length);	
		}
		else if ( uprobFilename==NULL && pprobFilename==NULL && reactFilename!=NULL )
			initialize_exp(reactFilename,'r',length);		
				
		
		if (TRACEBACK) {
			tracebackStructure = (int**)malloc(winSize*sizeof(int*));
			for (ts=0; ts<winSize; ts++) {
				tracebackStructure[ts] = (int*)malloc(winSize*sizeof(int));
				for (t=0; t<4; t++) {
					tracebackStructure[ts][(ts+t)%winSize] = -3;
				}	
			}
			tracebackChars = (char*)malloc(winSize*sizeof(char));
		}

		V = (float**)malloc((winSize+1)*sizeof(float*));
		W = (float**)malloc((winSize+1)*sizeof(float*));
		VBI = (float**)malloc((winSize+1)*sizeof(float*));
		VM = (float**)malloc((winSize+1)*sizeof(float*));

		Wcandidates = (int**)malloc((winSize+1)*sizeof(int*));
		VMcandidates = (float**)malloc((winSize+1)*sizeof(float*));
		lastWcand = (int*)malloc((winSize+1)*sizeof(int));

		for (k=0; k<(winSize+1) ;k++) {
			V[k] = (float*)malloc((winSize+1)*sizeof(float));
			W[k] = (float*)malloc((winSize+1)*sizeof(float));
			VBI[k] = (float*)malloc((winSize+1)*sizeof(float));
			VM[k] = (float*)malloc((winSize+1)*sizeof(float));
	
			Wcandidates[k] = (int*)malloc((winSize+1)*sizeof(int));
			VMcandidates[k] = (float*)malloc((winSize+1)*sizeof(float));

		}
		s.base = mallocate(winSize*sizeof(base));

		energy = optEnergy(str, winSize);
		fprintf(outp, ">%s\n", seqName);
		fprintf(outp,"%s\n",/*str*/subseq(str,0,winSize));
		if (TRACEBACK) {
			writeTraceback(outp, winSize);
		}
		fprintf(outp, "%lf\n", energy);
		
		winPoint = 1;
		for (k=1; k<endPoint ;k++) {
			energy = extendOptEnergy(str+k, winSize);

			fprintf(outp, ">%d\n", k);
			fprintf(outp,"%s\n",subseq(str,k,winSize+k-1));
			if (TRACEBACK) {
				writeTraceback(outp, winSize);
			}
			fprintf(outp,"\n");
			fprintf(outp, "%lf\n",energy);
			++winPoint;
		}
		
	}

	
	
	fclose(outp);	
	free_tables();
	for (k=0; k<(winSize+1) ;k++) {
		free(V[k]);
		free(W[k]);
		free(VBI[k]);
		free(VM[k]);

		free(Wcandidates[k]);
		free(VMcandidates[k]);
	}

	free(V);
	free(W);
	free(VBI);
	free(VM);

	free(Wcandidates);
	free(VMcandidates);
	free(lastWcand);
	
	free(str);
	free(s.base);

	return 0;
}

/* 
	Define path to program directory
*/
void pathHead(char* pathExec,char* head)
{
	char *root0 ,*root;
	int len0,len;
	
	len0=strrchr(pathExec,DIS)-pathExec;
	root0=(char*)malloc( (len0+1)*sizeof(char)  );
	strncpy(root0,pathExec, len0 );
	root0[len0]='\0';
	
	len=strrchr(root0,DIS)-root0;
	root=(char*)malloc( (len+1)*sizeof(char)  );
	strncpy(root,root0, len );
	root[len]='\0';
	strcpy(head,root);

	free(root0);
	free(root);
}
/*void pathHead(char* pathExec,char* head)
{
	char *root0 ,*root, *root1;
	int len0,len,len1;
	
	len0=strrchr(pathExec,DIS)-pathExec;
	root0=(char*)malloc( (len0+1)*sizeof(char)  );
	strncpy(root0,pathExec, len0 );
	root0[len0]='\0';
	
	len=strrchr(root0,DIS)-root0;
	root=(char*)malloc( (len+1)*sizeof(char)  );
	strncpy(root,root0, len );
	root[len]='\0';
	strcpy(head,root);
	
	len1=strrchr(root,DIS)-root;
	root1=(char*)malloc( (len1+1)*sizeof(char)  );
	strncpy(root1,root, len1 );
	root1[len1]='\0';
	strcpy(head,root1);

	free(root0);
	free(root);
	free(root1);
}
/*/

char* subseq(char* seq, int i, int j)
{
	char* ss=(char*)malloc( (j-i+2)*sizeof(char) );
	strncpy(ss,seq+i, j-i+1);
	return ss;

}