/*  
 * Authors: Ydo Wexler & Michal Ziv-Ukelson 2008
 * RNA secondary structure prediction of all consecutive sliding windows in a sequence.
 * The algorithm is based on the papers "A Study of Accessible Motifs and RNA Folding Complexity" 
 by Ydo Wexler, Chaya Zilberstein and Michal Ziv-Ukelson (JCB, 2007), 
 and the paper  "A fast algorithm to scan the RNA folding energy landscape of genomes 
 and its application to the analysis of genomic folding asymmetry" 
 by Yair Horesh, Ydo Wexler, Michal Ziv-Ukelson, and Ron Unger.

 Anyone using, modyfying, or redistributing this code is required to reference these works.
 Moreover, the copywrite notice must not be removed with any manipulation / redistribution of the code
 or the software.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *              
 */


#include "common.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


energy mins(energy a, energy b)
{
	if (a < b)
		return a;
	return b;
//  return (a < b ? a : b);
}

void *mallocate(int amount)
{
  void *addr = malloc(amount);

  if (addr == NULL)
    {
      fprintf(stderr, "Unable to allocate sufficient amount of memory\n");
      exit(2);
    }
  else
    return addr;
}

void *reallocate(void *addr, int amount)
{
  if ((addr = realloc(addr, amount)) == NULL)
    {
      fprintf(stderr, "Unable to reallocate sufficient amount of memory\n");
      exit(2);
    }
  else
    return addr;
}
