/* 
 * Authors: Ydo Wexler & Michal Ziv-Ukelson 2008
 * RNA secondary structure prediction of all consecutive sliding windows in a sequence.
 * The algorithm is based on the papers "A Study of Accessible Motifs and RNA Folding Complexity" 
 by Ydo Wexler, Chaya Zilberstein and Michal Ziv-Ukelson (JCB, 2007), 
 and the paper  "A fast algorithm to scan the RNA folding energy landscape of genomes 
 and its application to the analysis of genomic folding asymmetry" 
 by Yair Horesh, Ydo Wexler, Michal Ziv-Ukelson, and Ron Unger.

 Anyone using, modyfying, or redistributing this code is required to reference these works.
 Moreover, the copywrite notice must not be removed with any manipulation / redistribution of the code
 or the software.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *              
 */


#include "energy.h"
#include "../common.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

void init_bp() {

	int i, j;

	for (i=0; i<UNDEF + 1 ;i++) {
		for (j=0; j<UNDEF + 1 ;j++) {
			basepair[i][j] = MISMATCH;
		}
	}

	// now deal with the cases which are not mismatch
	basepair[A][U] = AU;
	basepair[C][G] = CG;
	basepair[G][C] = GC;
	basepair[G][U] = GU;
	basepair[U][A] = UA;
	basepair[U][G] = UG;

}
/*bp basepair(base a, base b)
{
  switch (a)
    {
    case A:
      if (b == U)
	return AU;
      else 
	return MISMATCH;
      break;
    case C:
      if (b == G)
	return CG;
      else
	return MISMATCH;
      break;
    case G:
      switch (b)
	{
	case C:
	  return GC;
	  break;
	case U:
	  return GU;
	  break;
	default:
	  return MISMATCH;
	}
      break;
    case U:
      switch (b)
	{
	case A:
	  return UA;
	  break;
	case G:
	  return UG;
	  break;
	default:
	  return MISMATCH;
	}
      break;
    default:
      return MISMATCH;
    }
	    
  return MISMATCH;
}*/

void test()
{
  int i, j;
  bp u, v;
  base a, b, c, d;

  printenergy(mloop_a);
  printf(", ");
  printenergy(mloop_b);
  printf(", ");
  printenergy(mloop_c);
  printf(", ");
  printenergy(ninio_max_correction);
  printf(", ");
  printf("%i, %i, ", short_stem_length, small_loop_size);
  printenergy(min_stacking);
  printf(", ");
  printenergy(size(loop_table_size + 10));
  printf("\n");

  for (i = 0; i <= loop_table_size; i++)
    {
      printenergy(loop[i]);
      printf(", ");
      printenergy(bulge[i]);
      printf(", ");
      printenergy(hairpin[i]);
      printf("\n");
    }

  for (i = 0; i < 6; i++)
    {
      for (j = 0; j < 6; j++)
	{
	  printenergy(ninio(i, j));
	  printf(", ");
	}
      printf("\n");
    }

  for (u = 0; u <= MISMATCH; u++)
    {
      for (v = 0; v <= MISMATCH; v++)
	{
	  printenergy(stack[u][v]);
	  printf(", ");
	}
      printf("\n");
    }

  for (u = 0; u <= MISMATCH; u++)
    for (a = 0; a <= UNDEF; a++)
      for (b = 0; b <= UNDEF; b++)
	{
	  for (c = 0; c <= UNDEF; c++)
	    for (d = 0; d <= UNDEF; d++)
	      {
		printenergy(tetraloop[u][a][b][c][d]);
		printf(", ");
	      }
	  printf("\n");
	}

  for (u = 0; u <= MISMATCH; u++)
    {
      for (a = 0; a <= UNDEF; a++)
	for (b = 0; b <= UNDEF; b++)
	  {
	    printenergy(hairpin_stacking[u][a][b]);
	    printf(", ");
	  }
      printf("\n");
    }

  for (u = 0; u <= MISMATCH; u++)
    {
      for (a = 0; a <= UNDEF; a++)
	for (b = 0; b <= UNDEF; b++)
	  {
	    printenergy(loop_stacking[u][a][b]);
	    printf(", ");
	  }
      printf("\n");
    }

  for (u = 0; u <= MISMATCH; u++)
    for (v = 0; v <= MISMATCH; v++)
      {
	for (a = 0; a <= UNDEF; a++)
	  for (b = 0; b <= UNDEF; b++)
	    {
	      printenergy(loop1x1[u][v][a][b]);
	      printf(", ");
	    }
	printf("\n");
      }
  for (u = 0; u <= MISMATCH; u++)
    for (v = 0; v <= MISMATCH; v++)
      {
	for (a = 0; a <= UNDEF; a++)
	  for (b = 0; b <= UNDEF; b++)
	    for (c = 0; c <= UNDEF; c++)
	      {
		printenergy(loop1x2[u][v][a][b][c]);
		printf(", ");
	      }
	printf("\n");
      }
  for (u = 0; u <= MISMATCH; u++)
    for (v = 0; v <= MISMATCH; v++)
      {
	for (a = 0; a <= UNDEF; a++)
	  for (b = 0; b <= UNDEF; b++)
	    for (c = 0; c <= UNDEF; c++)
	      for (d = 0; d <= UNDEF; d++)
		     {
		       printenergy(loop2x2[u][v][a][b][c][d]);
		       printf(", ");
		     }
	printf("\n");
      }

}

energy add_energies(energy a, energy b)
{
//#ifdef INTEGER

	if ((a >= INFINITEENERGY) || (b >= INFINITEENERGY)) {
		return INFINITEENERGY;
	}
	
	return a+b;

//  return (((a < INFINITEENERGY) && (b < INFINITEENERGY)) ? a + b : INFINITEENERGY);
//#else
 // return a + b;
//#endif
}

boolean1 equal_energies(energy a, energy b)
{
#ifdef INTEGER
  return (a == b);
#else
  return (-100*mins(a - b, b - a) <= -min_stacking);
#endif
}

energy times2(energy E)
{
  return 2 * E;
}

void printenergy(energy E)
{
#ifdef INTEGER
  if (E < 0)
    {
      printf("-");
      E = -E;
    }
  printf("%i.%i", E / 10, E % 10);
#else
  printf(ENERGYPRINTFORMAT, E);
#endif
}

/* Return the energy contributions from asymmetry and interior base pair/
 * unpaired bases stacking for interior loops.
 */
energy eL_noextstackingandsize(int i, int j, int k, int l, sequence s)
{
  return (basepair[s.base[l]][s.base[k]] != MISMATCH ?
	  add_energies
	  (loop_stacking[basepair[s.base[l]][s.base[k]]]
	   [s.base[l + 1]][s.base[k - 1]],
	   ninio(abs(k - i - j + l), mins(k - i, j - l) - 1))
	  : INFINITEENERGY);
}

/* Return the energy for interior loops. */
energy eL(int i, int j, int k, int l, sequence* s)
{
  if ((basepair[s->base[i]][s->base[j]] == MISMATCH)
       || (basepair[s->base[k]][s->base[l]] == MISMATCH))
    return INFINITEENERGY;

  if ((k > i + 1) && (j > l + 1))
    /* Real interior loop */
    if ((k - i + j - l > 6) || ((k - i + j - l == 6) && (k - i != 3)))
      /* Interior loop of size larger than 4 or 1x3/3x1 loop */
      return (add_energies
	      (add_energies
	       (loop_stacking[basepair[s->base[i]][s->base[j]]]
		[s->base[i + 1]][s->base[j - 1]],
		loop_stacking[basepair[s->base[l]][s->base[k]]]
		[s->base[l + 1]][s->base[k - 1]]),
	       add_energies
	       (ninio(abs(k - i - j + l), mins(k - i, j - l) - 1),
		size(k - i + j - l - 2))));
    else if (k - i + j - l == 6)
      /* 2x2 loop */
      return loop2x2[basepair[s->base[i]][s->base[j]]]
	[basepair[s->base[k]][s->base[l]]]
	[s->base[i + 1]][s->base[i + 2]][s->base[j - 1]][s->base[j - 2]];
    else if (k - i + j - l == 4)
      /* 1x1 loop */
      return loop1x1[basepair[s->base[i]][s->base[j]]]
	[basepair[s->base[k]][s->base[l]]]
	[s->base[i + 1]][s->base[j - 1]];
    else if (k == i + 2)
      /* 1x2 loop */
      return (loop1x2[basepair[s->base[i]][s->base[j]]]
	      [basepair[s->base[k]][s->base[l]]]
	      [s->base[i + 1]][s->base[j - 1]][s->base[j - 2]]);
    else
      /* 2x1 loop */
      return (loop1x2[basepair[s->base[l]][s->base[k]]]
	      [basepair[s->base[j]][s->base[i]]]
	      [s->base[j - 1]][s->base[i + 2]][s->base[i + 1]]);

  if ((k == i + 1) && (j == l + 1))
    /* Helix stacking */
	// es
    return 
      stack[basepair[s->base[i]][s->base[j]]][basepair[s->base[k]][s->base[l]]];

  if (k - i + j - l == 3)
    /* Bulge of size 1 */
    return add_energies
      (bulge[1],
       stack[basepair[s->base[i]][s->base[j]]][basepair[s->base[k]][s->base[l]]]);

  /* Bulge of size larger than 1 */
  if (k + j > loop_table_size + i + l + 2) {
	  return add_energies
	  (bulge[loop_table_size],
	   (energy)(logparam * log
		    ((float) (k - i + j - l - 2) / (float) loop_table_size)));
  }
  return bulge[k - i + j - l - 2];

/*  return (k + j > loop_table_size + i + l + 2 ?
	  add_energies
	  (bulge[loop_table_size],
	   (energy)(logparam * log
		    ((float) (k - i + j - l - 2) / (float) loop_table_size)))
	  : bulge[k - i + j - l - 2]);*/
}

/* Return unpaired base stacking effects for interior loops' exterior base
 * pair.
 */
energy eLstacking(int i, int j, sequence s)
{
  return loop_stacking[basepair[s.base[i]][s.base[j]]]
    [s.base[i + 1]][s.base[j - 1]];
}

/* Return the size contribution for interior loops */
energy size(int n)
{
  return (n > loop_table_size ? loop[loop_table_size]
	  + (energy)(logparam * log((float) n / (float) loop_table_size))
	  : loop[n]);
}

/* Return the Ninio asymmetry contribution */
energy ninio(int m, int n)
{
  return (mins(ninio_max_correction,
	      m * f[(n < short_stem_length ? n : short_stem_length)]));
}

/* Return the difference between asymmetry contributions for loops of size
 * n and n + 2 respectively.
 */
energy g(int n)
{
  /* With the Ninio correction the difference in asymmetry correction
   * is 0 when n >= short_stem_length; if n < short_stem_length the
   * prewrite method can't be used why t does not really matter what
   * is returned.
   */
  return 0;
}

/* Return the energy for hairpin loops */
energy eh(int i, int j, sequence s)
{
  energy E = hairpin[(j - i - 1 > loop_table_size) ?
		    loop_table_size : j - i - 1];

  if (basepair[s.base[i]][s.base[j]] == MISMATCH)
    return INFINITEENERGY;

  if (j - i > 4)
    {
      /* The following corrections does only apply to (some of the) hairpin
       * loops larger than size 3.
       */

      E = add_energies(E, hairpin_stacking[basepair[s.base[i]][s.base[j]]]
		       [s.base[i + 1]][s.base[j - 1]]);

      if (j - i - 1 > loop_table_size)
	E = add_energies
	  (E, (energy)(logparam
		       * log((float) (j - i - 1) / (float) loop_table_size)));
      else if (j - i == 5)
	E = add_energies(E, tetraloop[basepair[s.base[i]][s.base[j]]]
			 [s.base[i + 1]][s.base[i + 2]]
			 [s.base[i + 3]][s.base[i + 4]]);
    }

  return E;
}

/* Return energy of closing multibranched loop */
energy e(int i, int j, sequence s)
{
  return (basepair[s.base[i]][s.base[j]] != MISMATCH ? mloop_a + mloop_b
	  : INFINITEENERGY);
}

energy dangling5(int i, int j, sequence s)
{
  return dangle5[basepair[s.base[i]][s.base[j]]][s.base[i - 1]];
}

energy dangling3(int i, int j, sequence s)
{
  return dangle3[basepair[s.base[i]][s.base[j]]][s.base[j + 1]];
}

energy dangling3and5(int i, int j, sequence s)
{
  return add_energies(dangle5[basepair[s.base[i]][s.base[j]]][s.base[i - 1]],
		      dangle3[basepair[s.base[i]][s.base[j]]][s.base[j + 1]]);
}
