/* energy_read.c - read energy tables for RNA secondary structure prediction
 *                 Rune Lyngs
 *                 02/98
 */

//#include <values.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include "energy.h"
#include "../common.h"
#include "energy_read.h"

boolean1 energy_entry(char *s)
{
  int i;

  if (s == NULL)
    return FALSE;

  i = (s[0] == '-' || s[0] == '+' ? 1 : 0);
  while (isdigit(s[i]))
    i++;
  
  if (s[i] == '.')
    i++;
  
  while (isdigit(s[i]))
    i++;

  return ((i > 0) && (s[i] == 0) && ((s[i - 1] != '.') || (i == 1)));
}

#ifdef INTEGER
energy float2energy(float x)
{
  if (x < 0)
    return -((energy)(-10 * x + .5));
  else
    return (energy)(10 * x + .5);
}
#endif

void read_tables(/*void*/char* pathHead)
{
  FILE *datafile;
  char *line, *token, *token2, *directory, *extension, *filename, *envname;
  int i;
  float x, y, z;
  base a, b, c, d;
  bp u, v;
  base tetraloopbases[6];

  init_bp();

  /* Set the variable determining when a loop is large enough to be treated
   * as a general case.
   */
  small_loop_size = 5;

  /* Determine whether (parts of) the names of the energy files are
   * defined in the environment.
   */
  

  /* Allocate the strings that is used when reading in lines from the various
   * energy files and to construct the file names of the energy files.
   */
  line = mallocate(256*sizeof(char));
  directory = mallocate(256*sizeof(char));
  extension = mallocate(256*sizeof(char));
  filename = mallocate(256*sizeof(char));
  
  if ((envname = getenv("ZUKER_ENERGY_DIR")) != NULL)
    {
	  printf("envname: %s\n",envname);
      strcpy(directory, envname);
      strcat(directory, "/");
    }
  else
    //strcpy(directory, ENERGY_DIR);
	sprintf(directory,"%s%s",pathHead,ENERGY_DIR);
  if ((envname = getenv("ZUKER_ENERGY_EXT")) != NULL)
    {
      if (envname[0] != '.')
	{
	  strcpy(extension, ".");
	  strcat(extension, envname);
	}
      else strcpy(extension, envname);
    }
  else
    strcpy(extension, ENERGY_EXT);

  /* Read interior loop, bulge and hairpin loop values */

  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_LOOP_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, LOOP_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Find maximal size of loops tabulated */
  loop_table_size = 0;
  while (fgets(line, 256, datafile) != NULL)
    {
      if (sscanf(line, "%i "ENERGYSCANFORMAT" "ENERGYSCANFORMAT" "ENERGYSCANFORMAT, &i, &x, &y, &z) == 4)
	loop_table_size = (i > loop_table_size ? i : loop_table_size);
    }

  /* Allocate and initialize tables */
  loop = mallocate((loop_table_size + 1)*sizeof(energy));
  bulge = mallocate((loop_table_size + 1)*sizeof(energy));
  hairpin = mallocate((loop_table_size + 1)*sizeof(energy));

  for(i = 0; i <= loop_table_size; i++)
    loop[i] = bulge[i] = hairpin[i] = INFINITEENERGY;

  /* Reset LOOP_FILE and read tables */
  rewind(datafile);

  while (fgets(line, 256, datafile) != NULL)
    {
      if ((sscanf((char *)strtok(line, TOKEN_SEPARATOR), "%i", &i) == 1) && i <= loop_table_size)
	{
#ifdef INTEGER
       	  if (sscanf((char *)strtok(NULL, TOKEN_SEPARATOR),
		     ENERGYSCANFORMAT, &x) <= 0)
	    loop[i] = INFINITEENERGY;
	  else
	    loop[i] = float2energy(x);
	  if (sscanf((char *)strtok(NULL, TOKEN_SEPARATOR),
		     ENERGYSCANFORMAT, &x) <= 0)
	    bulge[i] = INFINITEENERGY;
	  else
	    bulge[i] = float2energy(x);
	  if (sscanf((char *)strtok(NULL, TOKEN_SEPARATOR),
		     ENERGYSCANFORMAT, &x) <= 0)
	    hairpin[i] = INFINITEENERGY;
	  else
	    hairpin[i] = float2energy(x);
#else
	  if (sscanf((char *)strtok(NULL, TOKEN_SEPARATOR),
		     ENERGYSCANFORMAT, loop + i) <= 0)
	    loop[i] = INFINITEENERGY;
	  if (sscanf((char *)strtok(NULL, TOKEN_SEPARATOR),
		     ENERGYSCANFORMAT, bulge + i) <= 0)
	    bulge[i] = INFINITEENERGY;
	  if (sscanf((char *)strtok(NULL, TOKEN_SEPARATOR),
		     ENERGYSCANFORMAT, hairpin + i) <= 0)
	    hairpin[i] = INFINITEENERGY;
#endif
	}
    }

  fclose(datafile);

  /* Find size of smallest hairpin loop possible */
  smallest_hairpin = 0;
  while ((smallest_hairpin < loop_table_size)
	 && (hairpin[smallest_hairpin] == INFINITEENERGY))
    smallest_hairpin++;

  /* Read dangling base stacking effect */
  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_DANGLE_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, DANGLE_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Read stacking effect for 3' dangling base */
  for (a = 0; a < UNDEF; a++)
    {
      /* Find next line of energy entries */
      do
	{
	  fgets(line, 256, datafile);
	  token = (char *)strtok(line, TOKEN_SEPARATOR);
	} while (!energy_entry(token));

      /* Read entries */
      for (b = 0; b < UNDEF; b++)
	{
	  for (c = 0; c < UNDEF; c++)
	    {
	      if ((basepair[b][a] == MISMATCH) || !strcmp(token, "."))
		dangle3[basepair[b][a]][c] = INFINITEENERGY;
	      else
#ifdef INTEGER
		{
		  sscanf(token, ENERGYSCANFORMAT, &x);
		  dangle3[basepair[b][a]][c] = float2energy(x);
		}
#else
	      sscanf(token, ENERGYSCANFORMAT, &dangle3[basepair[b][a]][c]);
#endif

	      token = (char *)strtok(NULL, TOKEN_SEPARATOR);
	    }

	  dangle3[basepair[b][a]][UNDEF] =
	    (basepair[b][a] == MISMATCH ? INFINITEENERGY : 0);
	}
    }

  /* Read stacking effects for 5' dangling base */
  for (a = 0; a < UNDEF; a++)
    {
      /* Find next line of energy entries */
      do
	{
	  fgets(line, 256, datafile);
	  token = (char *)strtok(line, TOKEN_SEPARATOR);
	} while (!energy_entry(token));

      /* Read entries */
      for (b = 0; b < UNDEF; b++)
	{
	  for (c = 0; c < UNDEF; c++)
	    {
	      if ((basepair[b][a] == MISMATCH) || !strcmp(token, "."))
		dangle5[basepair[b][a]][c] = INFINITEENERGY;
	      else
#ifdef INTEGER
		{
		  sscanf(token, ENERGYSCANFORMAT, &x);
		  dangle5[basepair[b][a]][c] = float2energy(x);
		}
#else
	      sscanf(token, ENERGYSCANFORMAT, &dangle5[basepair[b][a]][c]);
#endif

	      token = (char *)strtok(NULL, TOKEN_SEPARATOR);
	    }

	  dangle5[basepair[b][a]][UNDEF] =
	    (basepair[b][a] == MISMATCH ? INFINITEENERGY : 0);
	}
    }
  
  fclose(datafile);

  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_LOOP_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, MISCLOOP_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Read multiplicative constant for logarithmic energy-expansion of
   * large loops
   */
  do
    {
      fgets(line, 256, datafile);
      token = (char *)strtok(line, TOKEN_SEPARATOR);
    } while (!energy_entry(token));
  
  sscanf(token, ENERGYSCANFORMAT, &logparam);
#ifdef INTEGER
  /* With integer energies everything is in tenth of kcals */
  logparam *= 10.0;
#endif

  /* Read maximum asymmetry correction by Ninio rule */
  do
    {
      fgets(line, 256, datafile);
      token = (char *)strtok(line, TOKEN_SEPARATOR);
    } while (!energy_entry(token));

#ifdef INTEGER
  sscanf(token, ENERGYSCANFORMAT, &x);
  ninio_max_correction = float2energy(x);
#else
  sscanf(token, ENERGYSCANFORMAT, &ninio_max_correction);
#endif

  /* Read f array for Ninio rule */
  do
    {
      fgets(line, 256, datafile);
      token = (char *)strtok(line, TOKEN_SEPARATOR);
    } while (!energy_entry(token));

  f = mallocate(sizeof(energy));

  i = 0;
  f[0] = 0;

  do
    {
      i++;
      f = reallocate(f, (i + 1)*sizeof(energy));
      /* REALLOC(f, (i + 1) * sizeof(energy));*/
#ifdef INTEGER
      sscanf(token, ENERGYSCANFORMAT, &x);
      f[i] = float2energy(x);
#else
      sscanf(token, ENERGYSCANFORMAT, f + i);
#endif
      token = (char *)strtok(NULL, TOKEN_SEPARATOR);
    } while ((token != NULL) && energy_entry(token));

  /* Find value of short_stem_length */
  while ((i > 1) && (f[i] == f[i - 1]))
    i--;

  short_stem_length = i;

  /* Read constants for multibranched loops */
  do
    {
      fgets(line, 256, datafile);
      token = (char *)strtok(line, TOKEN_SEPARATOR);
    } while (!energy_entry(token));

#ifdef INTEGER
  sscanf(token, ENERGYSCANFORMAT, &x);
  mloop_a = float2energy(x);
  token = (char *)strtok(NULL, TOKEN_SEPARATOR);
  sscanf(token, ENERGYSCANFORMAT, &x);
  mloop_c = float2energy(x);
  token = (char *)strtok(NULL, TOKEN_SEPARATOR);
  sscanf(token, ENERGYSCANFORMAT, &x);
  mloop_b = float2energy(x);
#else
  sscanf(token, ENERGYSCANFORMAT, &mloop_a);
  token = (char *)strtok(NULL, TOKEN_SEPARATOR);
  sscanf(token, ENERGYSCANFORMAT, &mloop_c);
  token = (char *)strtok(NULL, TOKEN_SEPARATOR);
  sscanf(token, ENERGYSCANFORMAT, &mloop_b);
#endif

  fclose(datafile);

  /* Read duplex stacking values */
  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_STACK_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, STACK_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Read stacking effects */
  for (a = 0; a < UNDEF; a++)
    for (b = 0; b < UNDEF; b++)
      {
	/* Find next line of energy entries */
	do
	  {
	    fgets(line, 256, datafile);
	    token = (char *)strtok(line, TOKEN_SEPARATOR);
	  } while (!energy_entry(token));
	/* Read entries */
	for (c = 0; c < UNDEF; c++)
	  {
	    for (d = 0; d < UNDEF; d++)
	      {
		if ((basepair[b][d] == MISMATCH)
		    || (basepair[a][c] == MISMATCH)
		    || !strcmp(token, "."))
		  stack[basepair[a][c]][basepair[b][d]] = INFINITEENERGY;
		else
#ifdef INTEGER
		  {
		    sscanf(token, ENERGYSCANFORMAT, &x);
		    stack[basepair[a][c]][basepair[b][d]] =
		      float2energy(x);
		  }
#else
		  sscanf(token, ENERGYSCANFORMAT,
			 &stack[basepair[a][c]][basepair[b][d]]);
#endif
		token = (char *)strtok(NULL, TOKEN_SEPARATOR);
	      }
	  }
      }

  fclose(datafile);

  /* Read stibility bonus values for stable tetraloops */
  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_TLOOP_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, TLOOP_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Initialize tetraloop array */
  for (u = 0; u <= MISMATCH; u++)
    for(a = 0; a <= UNDEF; a++)
      for(b = 0; b <= UNDEF; b++)
	for(c = 0; c <= UNDEF; c++)
	  for(d = 0; d <= UNDEF; d++)
	    tetraloop[u][a][b][c][d] = 0;

  line = fgets(line, 256, datafile);
  do
    {
      /* Find lines with four bases followed by an energy value */
      token = (char *)strtok(line, TOKEN_SEPARATOR);
      token2 = (char *)strtok(NULL, TOKEN_SEPARATOR);	  
      if ((strlen(token) == 6) && energy_entry(token2))
	{
	  for (i = 0; i < 6; i++)
	    switch(token[i])
	      {
	      case 'A':
	      case 'a':
		tetraloopbases[i] = A;
		break;
	      case 'C':
	      case 'c':
		tetraloopbases[i] = C;
		break;
	      case 'G':
	      case 'g':
		tetraloopbases[i] = G;
		break;
	      case 'U':
	      case 'u':
		tetraloopbases[i] = U;
		break;
	      default:
		tetraloopbases[i] = UNDEF;
	      }
#ifdef INTEGER
	  sscanf(token2, ENERGYSCANFORMAT, &x);
	  tetraloop[basepair[tetraloopbases[0]][tetraloopbases[5]]]
	    [tetraloopbases[1]][tetraloopbases[2]]
	    [tetraloopbases[3]][tetraloopbases[4]] = float2energy(x);
#else
	  sscanf(token2, ENERGYSCANFORMAT,
		 &tetraloop[basepair[tetraloopbases[0]][tetraloopbases[5]]]
		 [tetraloopbases[1]][tetraloopbases[2]]
		 [tetraloopbases[3]][tetraloopbases[4]]);
#endif
	}
    } while (fgets(line, 256, datafile) != NULL);

  fclose(datafile);

  /* Read hairpin terminal unpaired base stacking effects */
  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_TSTACKH_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, TSTACKH_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Initialize array */
  for (a = 0; a <= UNDEF; a++)
    for (b = 0; b <= UNDEF; b++)
      {
	for (u = 0; u < MISMATCH; u++)
	  hairpin_stacking[u][a][b] = 0;

	hairpin_stacking[MISMATCH][a][b] = INFINITEENERGY;
      }

  /* Read stacking effects */
  for (a = 0; a < UNDEF; a++)
    for (b = 0; b < UNDEF; b++)
      {
	/* Find next line of energy entries */
	do
	  {
	    fgets(line, 256, datafile);
	    token = (char *)strtok(line, TOKEN_SEPARATOR);
	  } while (!energy_entry(token));
	/* Read entries */
	for (c = 0; c < UNDEF; c++)
	  {
	    for (d = 0; d < UNDEF; d++)
	      {
		if ((basepair[a][c] == MISMATCH)
		    || !strcmp(token, "."))
		  hairpin_stacking[basepair[a][c]][b][d] = INFINITEENERGY;
		else
#ifdef INTEGER
		  {
		    sscanf(token, ENERGYSCANFORMAT, &x);
		    hairpin_stacking[basepair[a][c]][b][d] = float2energy(x);
		  }
#else
		sscanf(token, ENERGYSCANFORMAT,
		       &hairpin_stacking[basepair[a][c]][b][d]);
#endif
		token = (char *)strtok(NULL, TOKEN_SEPARATOR);
	      }
	  }
      }

  fclose(datafile);

  /* Read interior loop terminal unpaired base stacking effect */
  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_TSTACKI_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, TSTACKI_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Initialize array */
  for (a = 0; a <= UNDEF; a++)
    for (b = 0; b <= UNDEF; b++)
      {
	for (u = 0; u < MISMATCH; u++)
	  loop_stacking[u][a][b] = 0;

	loop_stacking[MISMATCH][a][b] = INFINITEENERGY;
      }

  min_stacking = INFINITEENERGY;

  /* Read stacking effects */
  for (a = 0; a < UNDEF; a++)
    for (b = 0; b < UNDEF; b++)
      {
	/* Find next line of energy entries */
	do
	  {
	    fgets(line, 256, datafile);
	    token = (char *)strtok(line, TOKEN_SEPARATOR);
	  } while (!energy_entry(token));
	/* Read entries */
	for (c = 0; c < UNDEF; c++)
	  {
	    for (d = 0; d < UNDEF; d++)
	      {
		if ((basepair[a][c] == MISMATCH)
		    || !strcmp(token, "."))
		  loop_stacking[basepair[a][c]][b][d] = INFINITEENERGY;
		else
		  {
#ifdef INTEGER
		    sscanf(token, ENERGYSCANFORMAT, &x);
		    loop_stacking[basepair[a][c]][b][d] = float2energy(x);
#else
		    sscanf(token, ENERGYSCANFORMAT,
			   &loop_stacking[basepair[a][c]][b][d]);
#endif
		    min_stacking = mins(min_stacking, 
				       loop_stacking[basepair[a][c]][b][d]);
		  }
		token = (char *)strtok(NULL, TOKEN_SEPARATOR);
	      }
	  }
      }

  fclose(datafile);

  /* Read special values for small loops */
  /* Read special 1x1 interior loop stability values */
  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_SINT2_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, SINT2_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Initialize array */
  for (a = 0; a <= UNDEF; a++)
    for (b = 0; b <= UNDEF; b++)
      for (u = 0; u <= MISMATCH; u++)
	for (v = 0; v <= MISMATCH; v++)
	  loop1x1[u][v][a][b] = ((u != MISMATCH) && (v != MISMATCH) ?
				 0 : INFINITEENERGY);

  /* Read values */
  for (u = 0; u < MISMATCH; u++)
    for (a = 0; a < UNDEF; a++)
      {
	/* Find next line of energy entries */
	do
	  {
	    fgets(line, 256, datafile);
	    token = (char *)strtok(line, TOKEN_SEPARATOR);
	  } while (!energy_entry(token));
	/* Read entries */
	for (v = 0; v < MISMATCH; v++)
	  for (b = 0; b < UNDEF; b++)
	    {
	      if (!strcmp(token, "."))
		loop1x1[u][v][a][b] = INFINITEENERGY;
	      else
#ifdef INTEGER
		{
		  sscanf(token, ENERGYSCANFORMAT, &x);
		  loop1x1[u][v][a][b] = float2energy(x);
		}
#else
	      sscanf(token, ENERGYSCANFORMAT, &loop1x1[u][v][a][b]);
#endif
	      
	      token = (char *)strtok(NULL, TOKEN_SEPARATOR);
	    }
      }

  fclose(datafile);

  /* Read special 1x2 interior loop stability values */
  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_ASINT1X2_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, ASINT1X2_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Initialize array */
  for (a = 0; a <= UNDEF; a++)
    for (b = 0; b <= UNDEF; b++)
      for (c = 0; c <= UNDEF; c++)
	for (u = 0; u <= MISMATCH; u++)
	  for (v = 0; v <= MISMATCH; v++)
	    loop1x2[u][v][a][b][c] = ((u != MISMATCH) && (v != MISMATCH) ?
				      0 : INFINITEENERGY);
  /* Read values */
  for (u = 0; u < MISMATCH; u++)
    for (c = 0; c < UNDEF; c++)
      for (a = 0; a < UNDEF; a++)
	{
	  /* Find next line of energy entries */
	  do
	    {
	      fgets(line, 256, datafile);
	      token = (char *)strtok(line, TOKEN_SEPARATOR);
	    } while (!energy_entry(token));
	  /* Read entries */
	  for (v = 0; v < MISMATCH; v++)
	    for (b = 0; b < UNDEF; b++)
	      {
		if (!strcmp(token, "."))
		  loop1x2[u][v][a][b][c] = INFINITEENERGY;
		else
#ifdef INTEGER
		  {
		    sscanf(token, ENERGYSCANFORMAT, &x);
		    loop1x2[u][v][a][b][c] = float2energy(x);
		  }
#else
		sscanf(token, ENERGYSCANFORMAT, &loop1x2[u][v][a][b][c]);
#endif

		token = (char *)strtok(NULL, TOKEN_SEPARATOR);
	      }
	}

  fclose(datafile);

  /* Read special 2x2 interior loop stability values */
  strcpy(filename, directory);
  if ((envname = getenv("ZUKER_SINT4_FILE")) != NULL)
    strcat(filename, envname);
  else
    strcat(filename, SINT4_FILE);
  strcat(filename, extension);

  if ((datafile = fopen(filename, "r")) == NULL)
    {
      fprintf(stderr, "Could not open file %s\n", filename);
      exit(1);
    }

  /* Initialize array */
  for (a = 0; a <= UNDEF; a++)
    for (b = 0; b <= UNDEF; b++)
      for (c = 0; c <= UNDEF; c++)
	for (d = 0; d <= UNDEF; d++)
	  for (u = 0; u <= MISMATCH; u++)
	    for (v = 0; v <= MISMATCH; v++)
	      loop2x2[u][v][a][b][c][d] = ((u != MISMATCH) && (v != MISMATCH) ?
					   0 : INFINITEENERGY);

  /* Read values */
  for (u = 0; u < MISMATCH; u++)
    for (v = 0; v < MISMATCH; v++)
      for (a = 0; a < UNDEF; a++)
	for (c = 0; c < UNDEF; c++)
	  {
	    /* Find next line of energy entries */
	    do
	      {
		fgets(line, 256, datafile);
		token = (char *)strtok(line, TOKEN_SEPARATOR);
	      } while (!energy_entry(token));
	    /* Read entries */
	    for (b = 0; b < UNDEF; b++)
	      for (d = 0; d < UNDEF; d++)
		{
		  if (!strcmp(token, "."))
		    loop2x2[u][v][a][b][c][d] = INFINITEENERGY;
		  else
#ifdef INTEGER
		    {
		      sscanf(token, ENERGYSCANFORMAT, &x);
		      loop2x2[u][v][a][b][c][d] = float2energy(x);
		    }
#else
		  sscanf(token, ENERGYSCANFORMAT, &loop2x2[u][v][a][b][c][d]);
#endif

		  token = (char *)strtok(NULL, TOKEN_SEPARATOR);
		}
	  }
  
  fclose(datafile);


  /* free the allocated memeory */
  free(line);
  free(directory);
  free(extension);
  free(filename);
}
