/* rnaSlider.h 
 * Authors: Ydo Wexler & Michal Ziv-Ukelson 2008
 * RNA secondary structure prediction of all consecutive sliding windows in a sequence.
 * The algorithm is based on the papers "A Study of Accessible Motifs and RNA Folding Complexity" 
 by Ydo Wexler, Chaya Zilberstein and Michal Ziv-Ukelson (JCB, 2007), 
 and the paper  "A fast algorithm to scan the RNA folding energy landscape of genomes 
 and its application to the analysis of genomic folding asymmetry" 
 by Yair Horesh, Ydo Wexler, Michal Ziv-Ukelson, and Ron Unger.

 Anyone using, modyfying, or redistributing this code is required to reference these works.
 Moreover, the copywrite notice must not be removed with any manipulation / redistribution of the code
 or the software.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *              
 */


#ifndef INTERIOR

#define INTERIOR
/* APPLY_STOPRULE determines the minimum length sequences for which we start
 * to apply the stop rule. This constant can be changed to fine tune
 * performance. For short sequences (<750 bases) INT_MAX seems to be a good
 * choice.
 */
#define APPLY_STOPRULE 25
#define V(i, j) V[i][j]
#define W(i, j) W[i][j]

#include "common.h"
#include <stdio.h>

void preinterior(int k);
void multiloop(int i, int j);
void backtrackclosed(int i, int j, int indent);
energy backtrackmultiloop(int i, int j, int indent);
void allocate_tables(sequence s);
int main(int argc, char **argv);
energy optEnergy(char* str, int length);
void free_tables();
void printWV();
void findStructureTraceback(int length, int startpoint);
void writeTraceback(FILE* outp, int length);
//void initialize_pexp(char* filename);
//void initialize_uexp(char* filename);
#endif
void initialize_exp(char* filename,char type,int length);
double pcoef(int i);
double ucoef(int i, int j);
double rcoef(int i);
energy extendOptEnergy(char* str, int length);
void calculateParam();
void initializeEParam(int flag,char* paramfilename,int* usePar);
void finishParam();
double runSlider(char* str, double* ppaired, double* upaired, int wmax, int wmin, double intersect,char* argv);///

energy** V;
energy** W;
energy** VBI;
energy** VM;

int TRACEBACK;
int** tracebackStructure;
char* tracebackChars;

int** Wcandidates;
energy** VMcandidates;
int* lastWcand;
int winPoint;
int winSize;
int winMin;

double* pprob;
double* uprob;
double* react;
int pprob_exist;
int uprob_exist;
int react_exist;

double* expParam;
double* sdParam;
int* sizeParam;
/* percent of window intersection */
double winIntersect;
int makeParamCalculation;
sequence s;

int largest_loop;
/* exp_const = RT = 0.62 */
double exp_const;
//double uentropy,pentropy;