/*
main.c
Copyright (C) yyyy Ruslan Soldatov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "shuffle.h"
#include "shuffleFunc.h"
#include "methods.h"
#include "cmdline.h"

/* return sequence length */
int len(FILE* f)
{
	int i=0;
	char c;
	while ( feof(f)==0 )
	{
		c=fgetc(f);
		if ( convert(c)!=-1 ) i++;
		else if ( c!='\n' && c!='\r' && c!=' ' && c!='\t' && c!=-1 ) i++;
	}
	return i;
}


char* read_seq(FILE* file){
	char c;
	char* tmp;
	int length,i;
	fpos_t f_pos;
	
	if ( (c=fgetc(file)) != '>' ){
		fprintf(stderr,"Incorrect fasta-file");
		exit(1);
	}
	while ( (c=fgetc(file)) != '\n' && c!='\r' )
		;
	fgetpos(file,&f_pos);
	length=len(file);
	fsetpos(file,&f_pos);
	tmp=(char*)malloc(length*sizeof(char));
	i=0;
	while ( feof(file)==0 ){
		c=fgetc(file);
		if ( convert(c)!=-1 || (c!='\n' && c!='\r' && c!=' ' && c!='\t' && c!=-1) ){
			tmp[i]=c;
			i++;
		}
	}

	return tmp;
}


double* read_data(FILE* file,int length){
	double* tmp;
	int i;
	double x;
	tmp=(double*)malloc(length*sizeof(double));
	for (i=0;i<length;i++)
		tmp[i]=-1;
	while ( feof(file)==0 ){
		fscanf(file,"%d %lf",&i,&x);
		tmp[i-1]=x;
	}
	return tmp;
}

int main(int argc, char* argv[]){
	char *iname_seq,*iname_data,*iname_data1;
	char *oname_seq,*oname_data,*oname_data1;
	FILE *inp_seq,*inp_data,*inp_data1,*out_seq,*out_data,*out_data1;
	int di,markov,i;
	char* seq;
	double *data,*data1;
	struct gengetopt_args_info args_info;
	struct shuffleSeq newseq;
	di=0;
	markov=0;
	if (cmdline_parser(argc, argv, &args_info) != 0)
    {
		fprintf(stderr,"Wrong command line options\n");
		exit(1);
	}
	if ( args_info.input_given )
		iname_seq=args_info.input_arg;
	if ( args_info.data_given )
		iname_data=args_info.data_arg;
	if ( args_info.data1_given )
		iname_data1=args_info.data1_arg;
	if ( args_info.out_given )
		oname_seq=args_info.out_arg;
	if ( args_info.outdata_given )
		oname_data=args_info.outdata_arg;
	if ( args_info.outdata1_given )
		oname_data1=args_info.outdata1_arg;
	if ( args_info.di_given ) 
		di=1;
	if ( args_info.markov_given )
		markov=1;	
	if ( args_info.no_transform_given )
		qtransform=0;
	else qtransform=1;

	if ( (inp_seq = fopen(iname_seq, "r" )) == NULL ){
		fprintf(stderr,"Error in opening input file with sequence for read\n");
		exit( 1 );
	}	
	if ( args_info.data_given & (inp_data = fopen(iname_data, "r" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file with data for read\n");
		exit( 1 );
	}	
	if ( args_info.data1_given & (inp_data1 = fopen(iname_data1, "r" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file with data for read\n");
		exit( 1 );
	}
	if ( args_info.out_given & (out_seq = fopen(oname_seq, "w" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file for write\n");
		exit( 1 );
	}
	if ( args_info.outdata_given & (out_data = fopen(oname_data, "w" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file for write\n");
		exit( 1 );
	}
	if ( args_info.outdata1_given & (out_data1 = fopen(oname_data1, "w" )) == NULL )
	{
		fprintf(stderr,"Error in opening input file for write\n");
		exit( 1 );
	}

	/* read sequence and data */
	seq=read_seq(inp_seq);
	if ( args_info.data_given )
		data=read_data(inp_data,strlen(seq));
	if ( args_info.data1_given )
		data1=read_data(inp_data1,strlen(seq));

	srand(time(NULL));
	newseq=shuffle(seq,data,data1,di,markov);
	/* print sequence */
	if ( !args_info.out_given )
		out_seq=stdout;
	fprintf(out_seq,">shuffle\n");
	for (i=0;i<strlen(seq);i++)
		fprintf(out_seq,"%c",newseq.seq[i]);
	fprintf(out_seq,"\n");
		
	/* print data */
	if ( args_info.outdata_given ){
		for (i=0;i<strlen(seq);i++)
			fprintf(out_data,"%d %lf\n",(i+1),newseq.data[i]);
		fclose(out_data);
	}	
	if ( args_info.outdata1_given ){
		for (i=0;i<strlen(seq);i++)
			fprintf(out_data1,"%d %lf\n",(i+1),newseq.data1[i]);
		fclose(out_data1);
	}
	
	if ( !args_info.out_given )
		fclose(out_seq);
	if ( args_info.data_given )
		fclose(inp_data);
	if ( args_info.data1_given )
		fclose(inp_data1);
	if ( args_info.input_given )
		fclose(inp_seq);	
	return 0;
}

