/*
methods.c
Copyright (C) yyyy Ruslan Soldatov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/

#include "shuffleFunc.h"
#include "methods.h"


struct shuffleSeq emptyshuffleSeq(char* seq, double* data, double* data1){
	struct shuffleSeq tmp;
	int n;
	
	n=strlen(seq);
	tmp.seq=(char*)malloc( (n+1)*sizeof(char));
	if (data!=NULL)	{
		tmp.data_exists=1;
		tmp.data=(double*)malloc( (n+1)*sizeof(double));
	}
	else
		tmp.data_exists=0;
	if (data1!=NULL)	{
		tmp.data1_exists=1;
		tmp.data1=(double*)malloc( (n+1)*sizeof(double));
	}
	else
		tmp.data1_exists=0;
	return tmp;
}

/* Shuffle sequence with track */
struct shuffleSeq shuffleExact(char* seq, double* data, double* data1){
	struct shuffleSeq tmp;
	int* perm;
	int i,n;
	
	n=strlen(seq);
	perm=permutation(n);
	tmp=emptyshuffleSeq(seq, data, data1);
	for (i=0;i<n;i++){
		tmp.seq[i]=seq[ perm[i] ];
		if ( tmp.data_exists )
			tmp.data[i]=data[ perm[i] ];
		if ( tmp.data1_exists )
			tmp.data1[i]=data1[ perm[i] ];
	}
	tmp.seq[i]='\0';
	return tmp;
}

struct shuffleSeq shuffleMarkov(char* seq, double* data, double* data1){
	struct shuffleSeq tmp = emptyshuffleSeq(seq, data, data1);
	struct Info infotmp = makeInfo(seq,data, data1);
	double* weight;
	int i,k;
	
	weight=int2double(infotmp.content,4);
	for (i=0;i<infotmp.length;i++){
		k=rnd(4,weight);
		tmp.seq[i]=revert(k);
		if (tmp.data_exists==1){
			tmp.data[i]=infotmp.data.data[k][ rand() % infotmp.content[k] ];
		}
		if (tmp.data1_exists==1){
			tmp.data1[i]=infotmp.data.data1[k][ rand() % infotmp.content[k] ];
		}
	}
	tmp.seq[i]='\0';
	return tmp;
}


struct shuffleSeq shuffleDiMarkov(char* seq, double* data, double* data1){
	struct shuffleSeq tmp = emptyshuffleSeq(seq, data, data1);
	struct Info infotmp = makeInfo(seq,data, data1);
	double* weight;
	int i,k;
	
	weight=int2double(infotmp.content,4);
	k=rnd(4,weight);
	tmp.seq[0]=revert(k);
	if (tmp.data_exists==1){
		tmp.data[0]=infotmp.data.data[k][ rand() % infotmp.content[k] ];
	}
	if (tmp.data1_exists==1){
		tmp.data1[0]=infotmp.data.data1[k][ rand() % infotmp.content[k] ];
	}
	
	for (i=1;i<infotmp.length;i++){
		weight=int2double(infotmp.dicontent[k],4);
		k=rnd(4,weight);
		tmp.seq[i]=revert(k);
		if (tmp.data_exists==1){
			tmp.data[i]=infotmp.data.data[k][ rand() % infotmp.content[k] ];
		}
		if (tmp.data1_exists==1){
			tmp.data1[i]=infotmp.data.data1[k][ rand() % infotmp.content[k] ];
		}
	}
	tmp.seq[i]='\0';
	return tmp;
}

/* choose last edge for each graph vertex */
int* lastEdge(int last, struct Info info){
	int k;
	int* lstEdge=(int*)malloc(4*sizeof(int));
	
	lstEdge[last]=-1;
	for (k=0;k<4;k++){
		if (k!=last){
			lstEdge[k]=rnd(4, int2double(info.dicontent[k],4));
		}	
	}
	return lstEdge;
}

/* check whether lastage Edge set satisfy eularian path condition: 
	return 1 if eularian path, 0 - otherwise */
int eulearian(int last, int* lstEdge){
	int i,k,j,j1;
	int tmplstEdge[4];
	
	for (k=0;k<4;k++){
		if (k!=last){
			for (i=0;i<4;i++) tmplstEdge[i]=lstEdge[i];
			j=k;
			while ( tmplstEdge[j] != -1 ){
				j1=j;
				j=tmplstEdge[j];
				tmplstEdge[j1]=-1;
			}
			if (j!=last)
				return 0;
		}
	}
	return 1;
}

/* check whether vertex has out edges */
int empty(int k, struct Info info){
	int i;
	for (i=0;i<4;i++)
		if (info.dicontent[k][i]>0) return 1;
	return 0;
}


struct shuffleSeq shuffleDiExact(char* seq, double* data, double* data1){
	struct shuffleSeq tmp = emptyshuffleSeq(seq, data, data1);
	struct Info infotmp, infoshuf;
	int *perm,*lstEdge,ind[4];
	int i,j,k,knext,last,n;
	
	infotmp = makeInfo(seq,data, data1);
	infoshuf = makeInfo(seq,data, data1);
    //n=strlen(seq);
    //perm=permutation(n);
	/* shuffle data for each nt */
	if (tmp.data_exists==1){
		for (i=0;i<4;i++){
			perm=permutation(infotmp.content[i]);
			for (j=0;j<infotmp.content[i];j++){
				infoshuf.data.data[i][j]=infotmp.data.data[i][ perm[j] ];
                if (tmp.data1_exists==1)
                    infoshuf.data.data1[i][j]=infotmp.data.data1[i][ perm[j] ];
                //printf("%lf",infoshuf.data.data[i][ perm[j] ]);
			}
		}
	}
    /*
	if (tmp.data1_exists==1){
		for (i=0;i<4;i++){
			perm=permutation(infotmp.content[i]);
			for (j=0;j<infotmp.content[i];j++){
				infoshuf.data.data1[i][j]=infotmp.data.data1[i][ perm[j] ];
			}
		}
	}
     */
	/* choose lastage edges */
	last=convert(seq[ strlen(seq)-1 ]);
	lstEdge=lastEdge(last, infotmp);
	while ( eulearian(last,lstEdge)==0 ){
		lstEdge=lastEdge(last, infotmp);
	}

	/* exclude lastage edge from the graph */
	for (k=0;k<4;k++){
		if (k!=last)
			infoshuf.dicontent[k][lstEdge[k]]--;
	}
	
	/* fill shuffle sequence */
	for (i=0;i<4;i++) ind[i]=0;
	tmp.seq[0]=seq[0];
	if (tmp.data_exists) {
		k=convert(tmp.seq[0]);
		tmp.data[0]=infoshuf.data.data[k][ ind[k] ];
        if (tmp.data1_exists)
            tmp.data1[0]=infoshuf.data.data1[k][ ind[k] ];
		ind[k]++;
	}
    /*
	if (tmp.data1_exists) {
		k=convert(tmp.seq[0]);
		tmp.data1[0]=infoshuf.data.data1[k][ ind[k] ];
		ind[k]++;
	}
     */
	
	for (i=1; i<infotmp.length;i++){
		if ( empty(k,infoshuf)==0 && lstEdge[k]!=-1 ){
			knext=lstEdge[k];
			lstEdge[k]=-1;
		}
		else 
			knext=rnd(4, int2double(infoshuf.dicontent[k],4) );

		tmp.seq[i]=revert(knext);
		if (tmp.data_exists) tmp.data[i]=infoshuf.data.data[knext][ ind[knext] ];
		if (tmp.data1_exists) tmp.data1[i]=infoshuf.data.data1[knext][ ind[knext] ];
		ind[knext]++;
		infoshuf.dicontent[k][knext]--;
		k=knext;
	}
	 
	tmp.seq[i]='\0';
	return tmp;
}