/*
shuffleFunc.c
Copyright (C) yyyy Ruslan Soldatov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include "shuffleFunc.h"
#include "methods.h"

/* convert nucleotides to numbers (convert) and back (revert)
	A - 0
	T - 1
	G - 2
	C - 3
*/
int convert(char nt){
	if (nt=='A' || nt=='a') 
		return 0;
	else if (nt=='T' || nt=='t' || nt=='U' || nt=='u')
		return 1;
	else if (nt=='G' || nt=='g')
		return 2;
	else if (nt=='C' || nt=='c') 
		return 3;
	else{
		//fprintf(stderr,"Unknown character: %c\n",nt);
		return -1;
	}
}

char revert(int n){
	if (n==0)
		return 'a';
	else if (n==1)
		return 't';
	else if (n==2)
		return 'g';
	else if (n==3)
		return 'c';
	else return '?';
}

/* make structure Info based on sequence and data */
struct Info makeInfo(char* seq, double* data, double* data1){
	struct Info tempInfo;
	int i,j,k;
	i=0;
	tempInfo.length=strlen(seq);
	for (i=0;i<4;i++)
		tempInfo.content[i]=0;
	for (i=0;i<4;i++){
		for (j=0;j<4;j++)
			tempInfo.dicontent[i][j]=0;
	}
	i=0;
	while ( seq[i] ){
		k=convert(seq[i]);
		if (k!=-1){
			tempInfo.content[k]++;
			if (i!=0 && convert(seq[i-1])!=-1 )
				tempInfo.dicontent[convert(seq[i-1])][k]++;
		}	
		i++;
	}
	if (data!=NULL || data1!=NULL)
		tempInfo.data=makeData(seq,data,data1,tempInfo.content);
	return tempInfo;
}


/* make structure Data based on sequence and data */
struct Data makeData(char* seq, double* data, double* data1, int* content){
	struct Data tempData;
	int i,k;
	int ind[4];
	tempData.data=(double**)malloc(4*sizeof(double*));
	tempData.data1=(double**)malloc(4*sizeof(double*));
	for (i=0;i<4;i++){
		tempData.data[i]=(double*)malloc(content[i]*sizeof(double));
		tempData.data1[i]=(double*)malloc(content[i]*sizeof(double));
	}
	
	for (i=0;i<4;i++) ind[i]=0;
	i=0;
	while ( seq[i] ){
		k=convert(seq[i]);
		if (k!=-1){
			tempData.data[k][ind[k]]=data[i];
			tempData.data1[k][ind[k]]=data1[i];
			ind[k]++;
		}	
		i++;
		//ind[k]++;
	}
	return tempData;
}


/* return permutation of numbers 0,1,..,n-1 */
int* permutation(int n){
	int* tmp;
	int i,pos;
	
	tmp=(int*)malloc(n*sizeof(int));
	for (i=0;i<n;i++)
		tmp[i]=i;
	for (i=0;i<n;i++){
		pos = rand() % (n-i);
		swap(tmp,pos,n-i-1);
	}
	return tmp;
}


void swap(int* array, int i, int j){
	int k;
	k=array[i];
	array[i]=array[j];
	array[j]=k;
}


/* return random number from sample 0,1,..,n-1 with weights weight[0],..,weight[n-1] */
int rnd(int n, double* weight){
	double* ecdf;
	double weightSum,x;
	int i;
	
	ecdf=(double*)malloc(n*sizeof(double));
	weightSum=0;
	
	for (i=0;i<n;i++) 
		weightSum+=weight[i];
		
	for (i=0;i<n;i++){
		if (i==0) 
			ecdf[i]=weight[i]/weightSum;
		else 
			ecdf[i]=ecdf[i-1]+weight[i]/weightSum;
	}
	
	x=(double)rand()/RAND_MAX;
	i=0;
	while ( x>=ecdf[i] )
		i++;
	return i;
}

/* convert int array to double*/
double* int2double(int* array, int length){
	double* array1;
	int i;
	array1=(double*)malloc(length*sizeof(double));
	for (i=0;i<length;i++)
		array1[i]=(double)array[i];
	return array1;
}

void printSD(struct shuffleSeq obj){
	int i;
	i=0;
	printf("sequence:\n");
	i=0;
	while ( obj.seq[i] ){
		printf( "%c",obj.seq[i] );
		i++;
	}
	printf("\n");
	if (obj.data_exists == 1){
		printf("data:\n");
		i=0;
		while ( obj.seq[i] ){
			printf( "%lf ",obj.data[i] );
			i++;
		}
		printf("\n");
	}
}
