/*
transformation.h
Copyright (C) yyyy Ruslan Soldatov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include "transformation.h"


double expfit(double lam, int k){
	return exp(-lam*(k+1))/(1+exp(-lam));
}

double estLambda(int k){
	double threshold;
	double lam_left,lam_median,lam_right;
	threshold=(1-0.9)/2;
	lam_left=0;
	lam_right=100;
	lam_median=(lam_left + lam_right)/2;
	while (  fabs( expfit(lam_median,k)-threshold ) > threshold/100 ){
	    if ( expfit(lam_median,k) > threshold  )  
			lam_left=lam_median;
		else lam_right=lam_median;
		lam_median=(lam_left + lam_right)/2;
	}
	return lam_median;
}

/* copy int array 'from' to 'to' from position 'left' to 'right' */
void copyIntArray(int* from, int* to,int left, int right){
	int i;
	for (i=left;i<=right;i++)
		to[i]=from[i];
}

/* copy double array 'from' to 'to' from position 'left' to 'right' */
void copyDoubleArray(double* from, double* to,int left, int right){
	int i;
	for (i=left;i<=right;i++){
		to[i]=from[i];
	}
}

void split_merge(double* data_sorted, double* data_sorted_copy, int* ranks, int* ranks_copy, int left, int median, int right){
	int i0,i1,j;
	
	i0=left;
	i1=median+1;
	for (j=left;j<right+1;j++){
		if ( i0 <= median && (i1 > right || data_sorted[i0] <= data_sorted[i1])  ){
			data_sorted_copy[j]=data_sorted[i0];
			ranks_copy[j]=ranks[i0];
			i0++;
		}
		else{
			data_sorted_copy[j]=data_sorted[i1];
			ranks_copy[j]=ranks[i1];
			i1++;
		}
	}
}

/* function sort subarray */
void merge_subsort(double* data_sorted, double* data_sorted_copy, int* ranks, int* ranks_copy, int left, int right){
	int median,i;
	if ( right > left+1 ){
		median=(left+right)/2;
		merge_subsort(data_sorted,data_sorted_copy,ranks,ranks_copy,left,median);
		merge_subsort(data_sorted,data_sorted_copy,ranks,ranks_copy,(median+1),right);
		split_merge(data_sorted,data_sorted_copy,ranks,ranks_copy,left,median,right);
		copyDoubleArray(data_sorted_copy,data_sorted,left,right);
		copyIntArray(ranks_copy,ranks,left,right);
	}
	else if (right==left+1){
		if ( data_sorted[left]>data_sorted[right] ){
			data_sorted_copy[left]=data_sorted[right];
			data_sorted_copy[right]=data_sorted[left];
			ranks_copy[left]=ranks[right];
			ranks_copy[right]=ranks[left];
			copyDoubleArray(data_sorted_copy,data_sorted,left,right);
			copyIntArray(ranks_copy,ranks,left,right);
		}
	}
}


/* function sort array */
void merge_sort(int len, double* data, int* ranks, double* data_sorted){
	double* data_sorted_copy = (double*)malloc( len*sizeof(double));
	int* ranks_copy = (int*)malloc(len*sizeof(int));
	int i;
	copyIntArray(ranks,ranks_copy,0,len-1);
	copyDoubleArray(data,data_sorted,0,len-1);
	copyDoubleArray(data,data_sorted_copy,0,len-1);
	merge_subsort(data_sorted,data_sorted_copy,ranks,ranks_copy,0,len-1);
	free(data_sorted_copy);
	free(ranks_copy);
}

/* return autocorrelation data with its shift on k positions */
double autoCor(double data, int k){
	return 0;
}

void qtrans(int length, double* probs, double* shuffle_probs, double* shuffle_transform){
	int i;
	double *probs_sort,*shuffle_sort;
	int *ranks,*order;
	probs_sort=(double*)malloc(length*sizeof(double));
	shuffle_sort=(double*)malloc(length*sizeof(double));
	ranks=(int*)malloc(length*sizeof(int));
	order=(int*)malloc(length*sizeof(int));
	for (i=0;i<length;i++) 
		ranks[i]=i+1;
	merge_sort(length, probs, ranks, probs_sort);
	for (i=0;i<length;i++) 
		ranks[i]=i+1;
	merge_sort(length, shuffle_probs, ranks, shuffle_sort);
	for (i=0;i<length;i++){
		order[ ranks[i]-1 ] = i+1;
	}
	for (i=0;i<length;i++){
		shuffle_transform[i]=probs_sort[ order[i]-1 ];
	}
	
	free(probs_sort);
	free(shuffle_sort);
	free(ranks);
	free(order);
}


void smoothData(double* probs, double* probs_smooth, int length, int k,double lambda){
	int i,j;
	int start,end;
	double weight,w;
	
	for (i=0;i<length;i++){
		start=0;
		if ( i-k > start )
			start=i-k;
		end=length-1;
		if ( i+k < end )
			end=i+k;
		probs_smooth[i]=0;
		weight=0;
		for (j=start;j<end+1;j++){
			w=exp(-lambda*abs(i-j));
			probs_smooth[i]+=probs[j]*w;
			weight+=w;
		}
		probs_smooth[i]/=weight;	
	}
}

double avto(double* data, int length, int k){
	int i,iprev,j;
	double* data_sorted=(double*)malloc(length*sizeof(double));
	int* ranks=(int*)malloc(length*sizeof(int));
	int* ranks1=(int*)malloc(length*sizeof(int));
	double* order=(double*)malloc(length*sizeof(double));
	double spearman;
	
	for (i=0;i<length;i++)
		ranks[i]=i+1;
	merge_sort( length, data, ranks, data_sorted);
	
	for (i=0;i<length;i++){
		ranks1[ ranks[i]-1 ] = i+1;
	}
	iprev=0;
	for (i=1;i<length;i++){
		if ( data_sorted[i] != data_sorted[iprev] ){
			for (j=iprev;j<i;j++)
				order[j] = 1 + (double)(iprev+i-1)/2;
			iprev=i;
		}
	}
	for (j=iprev;j<length;j++)
		order[j] = 1 + (length-iprev)*(length+iprev-1)/2;
	
	spearman=0;
	for (i=k;i<length;i++){
		spearman+=( order[ranks1[i]-1] - order[ranks1[i-k]-1] )*( order[ranks1[i]-1] - order[ranks1[i-k]-1] );
	}
	spearman=1-(double)6*spearman/ (length-k)/( (length-k)*(length-k) - 1 );
	
	free(ranks);
	free(ranks1);
	free(order);
	free(data_sorted);
	return spearman;
}

int bestAuto(double* probs, double* shuffled, int length){
	int k;
	double true_cor,cor1,cor2;
	double* shuffled_smooth = (double*)malloc(length*sizeof(double));
	
	true_cor=avto( probs, length, 1);
	
	k=1;
	smoothData( shuffled, shuffled_smooth, length, k, estLambda(k));
	cor1=avto(shuffled_smooth,length,1);
	k=2;
	smoothData( shuffled, shuffled_smooth, length, k, estLambda(k));
	cor2=avto(shuffled_smooth,length,1);
	if ( true_cor < cor1 ){
		return 1;
	}
	else if ( true_cor >= cor1 && true_cor < cor2 ){
		if ( cor2 - true_cor < true_cor - cor1 )
			return 2;
		else return 1;
	}
	else{
		while ( !( true_cor >= cor1 && true_cor < cor2 ) ){
			cor1=cor2;
			k++;
			smoothData( shuffled, shuffled_smooth, length, k, estLambda(k));
			cor2=avto(shuffled_smooth,length,1);
		}
		if ( cor2 - true_cor < true_cor - cor1 )
			return k;
		else return (k-1);
	}
	
}


void transformation(double* probs, double* shuffle_probs, int length){
	double* shuffle_probs_smooth;
	int* ranks;
	int i,k;
	
	ranks=(int*)malloc(length*sizeof(int));
	for (i=0;i<length;i++)
		ranks[i]=i+1;
	shuffle_probs_smooth=(double*)malloc(length*sizeof(double));

	k=bestAuto(probs, shuffle_probs, length);
	smoothData(shuffle_probs, shuffle_probs_smooth, length, k, estLambda(k));
	qtrans(length, probs, shuffle_probs_smooth, shuffle_probs);
}


