/*
smooth.c
Copyright (C) yyyy Ruslan Soldatov

* RNASurface: fast and accurate detection of locally optimal potentially structured RNA segments

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Contact e-mail: ruslansoldatov@gmail.com
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#include "rnaSurface.h"


#define p_square 0.95
#define precision 0.0000001

float weight(int n,int i_pen);

/* 
	calculate empirical p_square
*/
double func(double q, int n)
{
	double numerator,denominator;
	numerator=4*exp(log(q)*(n+1))*(n+1-q*n)/(1-q)/(1-q);
	denominator=1+4*q/(1-q)/(1-q);
	return 1-numerator/denominator;
}

/* 
	calculate lambda 
*/
double calculateLambda(int n)
{
	double p_empiric,qLeft,qRight,qCenter;
	qLeft=0.0000001;
	qRight=1;
	qCenter=(qLeft+qRight)/2;
	p_empiric=func(qCenter,n);
	while ( ( p_square-p_empiric>precision ) || ( p_square-p_empiric<-precision ) )
	{
		if (p_empiric>p_square)	qLeft=qCenter;
		else	qRight=qCenter;
		qCenter=(qLeft+qRight)/2;
		p_empiric=func(qCenter,n);
	}
	return -log(qCenter);
}

/* 
	determine lambda
*/
void defLam(int n)
{
	lambda=calculateLambda(n);
}

void defStatWeight(int n)
{
	int i,j,k;
	int i_pen,j_pen;
	float sum;
	statWeight=(float*)malloc((n+1)*sizeof(float*));
	
	for (i_pen=0;i_pen<n+1;i_pen++)
	{
		statWeight[i_pen]=weight(n,i_pen);
	}
	
}

float weight(int n,int i_pen)
{
	int i,j,min;
	float sum;
	sum=0;
	for (i=-n;i<n+1;i++)
	{
		for (j=-n+abs(i);j<n-abs(i)+1;j++)
		{
			if (j-i>-n-1+i_pen) sum+=exp( -lambda*(abs(i)+abs(j)) );
		}
	}
	return sum;
}

/*
main()
{
	int n;
	scanf("%d",&n);
	printf("%lf\n",calculateLambda(n));
}*/